@@ -20,34 +20,45 @@ MIN_PWD_LEN = os.getenv("MIN_PWD_LEN", 8)
 
 
 class SecretDetector(logging.Formatter):
-    AWS_KEY_PATTERN = re.compile(r"(aws_key_id|aws_secret_key|access_key_id|secret_access_key)\s*=\s*'([^']+)'",
-                                 flags=re.IGNORECASE)
-    AWS_TOKEN_PATTERN = re.compile(r'(accessToken|tempToken|keySecret)"\s*:\s*"([a-z0-9/+]{32,}={0,2})"',
-                                   flags=re.IGNORECASE)
-    SAS_TOKEN_PATTERN = re.compile(r'(sig|signature|AWSAccessKeyId|password|passcode)=(?P<secret>[a-z0-9%/+]{16,})',
-                                   flags=re.IGNORECASE)
-    PRIVATE_KEY_PATTERN = re.compile(r'-----BEGIN PRIVATE KEY-----\\n([a-z0-9/+=\\n]{32,})\\n-----END PRIVATE KEY-----',
-                                     flags=re.MULTILINE | re.IGNORECASE)
-    PRIVATE_KEY_DATA_PATTERN = re.compile(r'"privateKeyData": "([a-z0-9/+=\\n]{10,})"',
-                                          flags=re.MULTILINE | re.IGNORECASE)
-    CONNECTION_TOKEN_PATTERN = re.compile(r'(token|assertion content)'
-                                          r'([\'\"\s:=]+)'
-                                          r'([a-z0-9=/_\-\+]{8,})',
-                                          flags=re.IGNORECASE)
+    AWS_KEY_PATTERN = re.compile(
+        r"(aws_key_id|aws_secret_key|access_key_id|secret_access_key)\s*=\s*'([^']+)'",
+        flags=re.IGNORECASE,
+    )
+    AWS_TOKEN_PATTERN = re.compile(
+        r'(accessToken|tempToken|keySecret)"\s*:\s*"([a-z0-9/+]{32,}={0,2})"',
+        flags=re.IGNORECASE,
+    )
+    SAS_TOKEN_PATTERN = re.compile(
+        r"(sig|signature|AWSAccessKeyId|password|passcode)=(?P<secret>[a-z0-9%/+]{16,})",
+        flags=re.IGNORECASE,
+    )
+    PRIVATE_KEY_PATTERN = re.compile(
+        r"-----BEGIN PRIVATE KEY-----\\n([a-z0-9/+=\\n]{32,})\\n-----END PRIVATE KEY-----",
+        flags=re.MULTILINE | re.IGNORECASE,
+    )
+    PRIVATE_KEY_DATA_PATTERN = re.compile(
+        r'"privateKeyData": "([a-z0-9/+=\\n]{10,})"', flags=re.MULTILINE | re.IGNORECASE
+    )
+    CONNECTION_TOKEN_PATTERN = re.compile(
+        r"(token|assertion content)" r"([\'\"\s:=]+)" r"([a-z0-9=/_\-\+]{8,})",
+        flags=re.IGNORECASE,
+    )
 
-    PASSWORD_PATTERN = re.compile(r'(password'
-                                  r'|pwd)'
-                                  r'([\'\"\s:=]+)'
-                                  r'([a-z0-9!\"#\$%&\\\'\(\)\*\+\,-\./:;<=>\?\@\[\]\^_`\{\|\}~]{8,})',
-                                  flags=re.IGNORECASE)
+    PASSWORD_PATTERN = re.compile(
+        r"(password"
+        r"|pwd)"
+        r"([\'\"\s:=]+)"
+        r"([a-z0-9!\"#\$%&\\\'\(\)\*\+\,-\./:;<=>\?\@\[\]\^_`\{\|\}~]{8,})",
+        flags=re.IGNORECASE,
+    )
 
     @staticmethod
     def mask_connection_token(text):
-        return SecretDetector.CONNECTION_TOKEN_PATTERN.sub(r'\1\2****', text)
+        return SecretDetector.CONNECTION_TOKEN_PATTERN.sub(r"\1\2****", text)
 
     @staticmethod
     def mask_password(text):
-        return SecretDetector.PASSWORD_PATTERN.sub(r'\1\2****', text)
+        return SecretDetector.PASSWORD_PATTERN.sub(r"\1\2****", text)
 
     @staticmethod
     def mask_aws_keys(text):
@@ -55,7 +66,7 @@ class SecretDetector(logging.Formatter):
 
     @staticmethod
     def mask_sas_tokens(text):
-        return SecretDetector.SAS_TOKEN_PATTERN.sub(r'\1=****', text)
+        return SecretDetector.SAS_TOKEN_PATTERN.sub(r"\1=****", text)
 
     @staticmethod
     def mask_aws_tokens(text):
@@ -64,11 +75,14 @@ class SecretDetector(logging.Formatter):
     @staticmethod
     def mask_private_key(text):
         return SecretDetector.PRIVATE_KEY_PATTERN.sub(
-            "-----BEGIN PRIVATE KEY-----\\\\nXXXX\\\\n-----END PRIVATE KEY-----", text)
+            "-----BEGIN PRIVATE KEY-----\\\\nXXXX\\\\n-----END PRIVATE KEY-----", text
+        )
 
     @staticmethod
     def mask_private_key_data(text):
-        return SecretDetector.PRIVATE_KEY_DATA_PATTERN.sub('"privateKeyData": "XXXX"', text)
+        return SecretDetector.PRIVATE_KEY_DATA_PATTERN.sub(
+            '"privateKeyData": "XXXX"', text
+        )
 
     @staticmethod
     def mask_secrets(text: str) -> Tuple[bool, str, str]:
@@ -92,9 +106,7 @@ class SecretDetector(logging.Formatter):
                         SecretDetector.mask_private_key(
                             SecretDetector.mask_aws_tokens(
                                 SecretDetector.mask_sas_tokens(
-                                    SecretDetector.mask_aws_keys(
-                                        text
-                                    )
+                                    SecretDetector.mask_aws_keys(text)
                                 )
                             )
                         )
@@ -126,16 +138,25 @@ class SecretDetector(logging.Formatter):
         """
         try:
             unsanitized_log = super().format(record)
-            masked, sanitized_log, err_str = \
-                SecretDetector.mask_secrets(unsanitized_log)
+            masked, sanitized_log, err_str = SecretDetector.mask_secrets(
+                unsanitized_log
+            )
             if masked and err_str is not None:
                 sanitized_log = "{} - {} {} - {} - {} - {}".format(
-                    record.asctime, record.threadName,
-                    "secret_detector.py", "sanitize_log_str",
-                    record.levelname, err_str)
+                    record.asctime,
+                    record.threadName,
+                    "secret_detector.py",
+                    "sanitize_log_str",
+                    record.levelname,
+                    err_str,
+                )
         except Exception as ex:
             sanitized_log = "{} - {} {} - {} - {} - {}".format(
-                record.asctime, record.threadName,
-                "secret_detector.py", "sanitize_log_str",
-                record.levelname, "EXCEPTION - " + str(ex))
+                record.asctime,
+                record.threadName,
+                "secret_detector.py",
+                "sanitize_log_str",
+                record.levelname,
+                "EXCEPTION - " + str(ex),
+            )
         return sanitized_log
